/*!
 * @license
 * 
 * candle.js v0.2.5
 *  https://bitbucket.org/sabo2/candle
 * 
 * This script is referencing following library.
 *  uuCanvas.js (version 1.0)
 *  http://code.google.com/p/uupaa-js-spinoff/
 * 
 * Copyright 2011-2014 sabo2
 * 
 * This script is released under the MIT license. Please see below.
 *  http://www.opensource.org/licenses/mit-license.php
 * 
 * Date: 2014-03-23
 */

// intro.js

(function(){

// 多重定義防止
if(!!window.Candle){ return;}

// candle.core.js

/* ------------- */
/*   variables   */
/* ------------- */
var _doc = document,
	_2PI = 2*Math.PI,
	_color = [],
	_css = [];

/* ---------- */
/*   arrays   */
/* ---------- */
var _hex = (function(){
	var tbl = [];
	for(var r=256;r<512;r++){ tbl[r-256]=r.toString(16).substr(1);}
	return tbl;
})();

/* ---------------------- */
/*   Candleオブジェクト   */
/* ---------------------- */
var Candle = {
	version: "0.2.5",
	
	/* wrapper classes */
	wrapper : {},
	addWrapper : function(classname, proto){
		classname = classname.replace(/\s+/g,'');
		var colon = classname.indexOf(':'), basename = '';
		if(colon>=0){
			basename  = classname.substr(colon+1);
			classname = classname.substr(0,colon);
		}

		var NewClass = function(){ if(!!this.initialize){ this.initialize.apply(this,arguments);}};
		if(!!basename && !!this.wrapper[basename]){
			var BaseClass = this.wrapper[basename];
			for(var name in BaseClass.prototype){ NewClass.prototype[name] = BaseClass.prototype[name];}
		}
		for(var name in proto){ NewClass.prototype[name] = proto[name];}
		NewClass.prototype.constructor = NewClass;
		var rel = {body:NewClass, name:classname, base:basename};
		this.wrapper[rel.name] = rel.body;
	},

	/* TypeList class */
	_order : [],
	enable : {},
	addTypes : function(type){
		this._order.push(type);
		this.enable[type] = true;
		if(!this.current){ this.current=type;}
	},

	TypeList : function(type){
		for(var i=0;i<Candle._order.length;i++){
			this[Candle._order[i]]=(Candle._order[i]===type);
		}
	},

	/* Selected & Enable types */
	current : '',
	select : function(type){
		if(!this.enable[type]){ return false;}
		this.current = type;
		return true;
	},

	/* externs */
	ME     : null,
	initME : function(){
		var me = _doc.createElement('div');
		me.style.display  = 'inline';
		me.style.position = 'absolute';
		me.style.left     = '-9000px';
		me.innerHTML = '';
		_doc.body.appendChild(me);

		this.ME = me;
	},

	/* color parser */
	color : _color,
	parse : function(rgbstr){
		if(!_color[rgbstr]){
			if(rgbstr.substr(0,4)==='rgb('){
				var m = rgbstr.match(/\d+/g);
				_color[rgbstr] = ["#",_hex[m[0]],_hex[m[1]],_hex[m[2]]].join('');
			}
			else{ _color[rgbstr] = rgbstr;}
		}
		return _color[rgbstr];
	},
	parsecolorrev : function(colorstr){
		if(colorstr.match(/\#([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])/)){
			var m0 = parseInt(RegExp.$1,16).toString();
			var m1 = parseInt(RegExp.$2,16).toString();
			var m2 = parseInt(RegExp.$3,16).toString();
			return ["rgb(",m0,',',m1,',',m2,")"].join('');
		}
		return colorstr;
	},

	/* DOM datas */
	getRectSize : function(el){
		return { width :(el.offsetWidth  || el.clientWidth),
				 height:(el.offsetHeight || el.clientHeight)};
	},

	/* functions */
	_counter : -1,
	getcanvasid : function(){
		this._counter++;
		return "_candle_canvas_"+this._counter;
	},

	_initializing : 0,
	initAllElements : function(){
		var elements = _doc.getElementsByTagName('candle');
		for(var i=0;i<elements.length;i++){ this.start(elements[i].id);}
	},
	start : function(idname, type, initCallBack){
		var el = _doc.getElementById(this.EL_ID_HEADER + idname);
		if(!!el){
			if(this.readyflag[idname]===true){
				initCallBack(el.parentNode.getContext('2d'));
			}
			return;
		}
		if(!this.ME){ this.initME();}

		var choice = type;
		if(!this.enable[choice]){ choice=this.current;}
		if(!this.enable[choice]){ return;}

		this._initializing++;
		this.readyflag[idname] = false;
		var context = new this.wrapper[choice](idname), self = this;

		if(!!context && !!initCallBack){
			if(self.isready(idname)){ initCallBack(context);}
			else{
				setTimeout(function(){
					if(!self.isready(idname)){
						setTimeout(arguments.callee,20);
						return;
					}
					initCallBack(context);
				},20);
			}
		}
	},

	readyflag : {},
	isready : function(idname){ return !!this.readyflag[idname];},
	allready : function(){ return (this._initializing===0);},

	debugmode : false,

	/* initialize functions */
	onload : function(){
		this.createCSS();
		this.initAllElements();
	},

	sheet : null,
	createCSS : function(){
		if(!!this.sheet){ return;}
		var _head = _doc.getElementsByTagName('head')[0];
		if(!!_head){
			var style = _doc.createElement('style');
			style.setAttribute('type', "text/css");
			_head.appendChild(style);
		}
		else{ _doc.write("<style></style>");}
		this.sheet = _doc.styleSheets[_doc.styleSheets.length - 1];
		for(var i=0;i<_css.length;i++){ this.addCSS(_css[i][0],_css[i][1]);}
		_css=[];
	},
	addCSS : function(sel,rule){
		if(!!this.sheet){
			var s = this.sheet;
			if(!!s.insertRule){ s.insertRule(sel+'{'+rule+'}',s.cssRules.length);}
			else if(!!s.addRule){ s.addRule(sel,rule,-1);}
		}
		else{ _css.push(sel,rule);}
	}
};

// 初期化関数設定 
if(!!window.addEventListener){ window.addEventListener("load",function(){ Candle.onload();},false);}
else if(!!window.attachEvent){ window.attachEvent("onload",function(){ Candle.onload();});}

// IE用ハック
_doc.createElement('candle');

// CSS設定 
Candle.createCSS();

// extern
window.Candle = Candle;

// candle.base.js
 
/* ------------------- */
/*  WrapperBaseクラス  */
/* ------------------- */
Candle.addWrapper('wrapperbase',{
	initialize : function(idname){
		// canvasに存在するプロパティ＆デフォルト値
		this.fillStyle    = 'black';
		this.strokeStyle  = 'black';
		this.lineWidth    = 1;
		this.font         = '14px system';
		this.textAlign    = 'center';
		this.textBaseline = 'middle';
		this.canvas = null;		// 親エレメントとなるdivエレメント

		// variables for internal
		this.idname   = idname;
		this.canvasid = Candle.getcanvasid();
		this.child    = null;	// 親エレメントの直下にあるエレメント

		// Layer additional
		this.currentLayerId = '_empty';
		this.isedgearray    = {_empty:false};
		this.isedge         = false;
	}
});

/* ----------------------- */
/*   VectorContextクラス   */
/* ----------------------- */
Candle.addWrapper('vector:wrapperbase',{

	initialize : function(idname){
		Candle.wrapper.wrapperbase.prototype.initialize.call(this, idname);

		// 外部から変更される追加プロパティ
		this.vid      = '';
		this.elements = [];
		this.lastElement = null;

		// variables for internal
		this.target = null;	// エレメントの追加対象となるオブジェクト

		// 描画中path
		this.cpath    = [];
		this.lastpath = '';
	},

	/* additional functions (for initialize) */
	initElement : function(){},
	afterInit : function(){
		var parent = this.canvas;
		var child  = this.child;
		var rect   = Candle.getRectSize(parent);

		var self = this;
		parent.style.overflow = 'hidden';
		if(Candle.debugmode){
			parent.style.backgroundColor = "#efefef";
			parent.style.border = "solid 1px silver";
		}
		parent.getContext = function(type){ return self;};
		parent.toDataURL = function(type){ return null; /* 未サポート */ };
		parent.toBlob = function(){ return null; /* 未サポート */ };

		this.target = this.child;
		this.rect(0,0,rect.width,rect.height);
		this.addVectorElement(false,false);

		this.setLayer();

		Candle._initializing--;
		Candle.readyflag[this.idname] = true;
	},

	initTarget : function(){},
	clear : function(){
		this.resetElement();
	},
	resetElement : function(){
		this.vid = '';
		this.elements = [];
		this.lastElement = null;
		this.initTarget();
	},

	/* layer functions */
	setLayer : function(layerid){
		this.initTarget();
		if(!!layerid){
			var lid = [this.canvasid,"layer",layerid].join('_');
			var layer = this.getLayerById(lid);
			if(!layer){ layer = this.createLayer(lid);}
			this.target = layer;
		}
		this.currentLayerId = (!!layerid ? layerid : '_empty');
		this.setLayerEdge();
	},

	setLayerEdge : function(){
		if(this.isedgearray[this.currentLayerId] !== void 0)
			{ this.isedge = this.isedgearray[this.currentLayerId];}
		else
			{ this.isedge = this.isedgearray['_empty'];}
	},
	getLayerById : function(){},
	createLayer : function(lid){ return null;},

	/* property functions */
	setRendering : function(render){},
	setUnselectable : function(unsel){},

	getContextElement : function(){ return this.child;},
	getLayerElement   : function(){ return this.target;},

	changeSize : function(width,height){
		this.canvas.style.width  = width + 'px';
		this.canvas.style.height = height + 'px';

		this.changeChildSize(this.canvas.firstChild,width,height);
	},
	changeChildSize : function(child,width,height){},

	/* Canvas API functions (for path) */
	beginPath : function(){
		this.cpath = [];
		this.lastpath = '';
	},
	closePath : function(){
		this.cpath.push(this.PATH_CLOSE);
		this.lastpath = this.PATH_CLOSE;
	},

	moveTo : function(x,y){
		this.cpath.push(this.PATH_MOVE,x,y);
		this.lastpath = this.PATH_MOVE;
	},
	lineTo : function(x,y){
		if(this.lastpath!==this.PATH_LINE){ this.cpath.push(this.PATH_LINE);}
		this.cpath.push(x,y);
		this.lastpath = this.PATH_LINE;
	},
	rect : function(x,y,w,h){
		this.cpath.push(this.PATH_MOVE,x,y,this.PATH_LINE,(x+w),y,(x+w),(y+h),x,(y+h),this.PATH_CLOSE);
		this.lastpath = this.PATH_CLOSE;
	},
	arc : function(cx,cy,r,startRad,endRad,antiClockWise){
		var sx,sy,ex,ey;
		if(endRad-startRad>=_2PI){ sx=cx+r, sy=cy, ex=cx+r, ey=cy;}
		else{
			sx = cx + r*Math.cos(startRad), sy = cy + r*Math.sin(startRad),
			ex = cx + r*Math.cos(endRad),   ey = cy + r*Math.sin(endRad);
		}
		if(endRad-startRad>=_2PI){ sy+=0.125;}
		var unknownflag = (startRad>endRad)^(Math.abs(endRad-startRad)>Math.PI);
		var islong = ((antiClockWise^unknownflag)?1:0), sweep = ((islong==0^unknownflag)?1:0);
		this.cpath.push(this.PATH_MOVE,sx,sy,this.PATH_ARCTO,r,r,0,islong,sweep,ex,ey);
		this.lastpath = this.PATH_ARCTO;
	},

	/* Canvas API functions (for drawing) */
	fill   : function(){ this.addVectorElement(true,false);},
	stroke : function(){ this.addVectorElement(false,true);},
	shape  : function(){ this.addVectorElement(true,true);}, /* extension */

	/* Canvas API functions (rect) */
	fillRect   : function(x,y,w,h){
		var stack = this.cpath;
		this.cpath = [];
		this.rect(x,y,w,h);
		this.addVectorElement(true,false);
		this.cpath = stack;
	},
	strokeRect : function(x,y,w,h){
		var stack = this.cpath;
		this.cpath = [];
		this.rect(x,y,w,h);
		this.addVectorElement(false,true);
		this.cpath = stack;
	},
	shapeRect  : function(x,y,w,h){
		var stack = this.cpath;
		this.cpath = [];
		this.rect(x,y,w,h);
		this.addVectorElement(true,true);
		this.cpath = stack;
	},

	/* Canvas API functions (for text) */
	fillText : function(text,x,y){},

	/* Canvas API functions (for image) */
	drawImage : function(image,sx,sy,sw,sh,dx,dy,dw,dh){},

	/* Canvas API functions (for transform) */
	translate : function(left,top){},

	/* extended functions */
	setLinePath : function(){
		var _args=arguments, _len=_args.length, len=_len-((_len|1)?1:2), a=[];
		for(var i=0;i<len;i+=2){ a[i>>1] = [_args[i],_args[i+1]];}
		this.setLinePath_com.call(this,a);
		if(_args[_len-1]){ this.cpath.push(this.PATH_CLOSE);}
	},
	setOffsetLinePath : function(){
		var _args=arguments, _len=_args.length, len=_len-((_len|1)?1:2), a=[];
		for(var i=0;i<len-2;i+=2){ a[i>>1] = [_args[i+2]+_args[0], _args[i+3]+_args[1]];}
		this.setLinePath_com.call(this,a);
		if(_args[_len-1]){ this.cpath.push(this.PATH_CLOSE);}
	},
	setLinePath_com : function(array){
		this.cpath = [];
		for(var i=0,len=array.length;i<len;i++){
			this.cpath.push(i===0 ? this.PATH_MOVE : this.PATH_LINE);
			this.cpath.push(array[i][0],array[i][1]);
		}
	},
	setDashSize : function(sizes){},

	strokeLine : function(x1,y1,x2,y2){
		var stack = this.cpath;
		this.cpath = [this.PATH_MOVE,x1,y1,this.PATH_LINE,x2,y2];
		this.addVectorElement(false,true);
		this.cpath = stack;
	},
	strokeDashedLine : function(x1,y1,x2,y2,sizes){
		var stack = this.cpath;
		this.cpath = [this.PATH_MOVE,x1,y1,this.PATH_LINE,x2,y2];
		this.addVectorElement(false,true);
		this.setDashSize(sizes);
		this.cpath = stack;
	},
	strokeCross : function(cx,cy,l){
		var stack = this.cpath;
		this.cpath = [this.PATH_MOVE,(cx-l),(cy-l),this.PATH_LINE,(cx+l),(cy+l),
					  this.PATH_MOVE,(cx-l),(cy+l),this.PATH_LINE,(cx+l),(cy-l)];
		this.addVectorElement(false,true);
		this.cpath = stack;
	},

	/* extended functions (circle) */
	fillCircle : function(cx,cy,r){
		var stack = this.cpath;
		this.cpath = [];
		this.arc(cx,cy,r,0,_2PI,false);
		this.cpath.push(this.PATH_CLOSE);
		this.addVectorElement(true,false);
		this.cpath = stack;
	},
	strokeCircle : function(cx,cy,r){
		var stack = this.cpath;
		this.cpath = [];
		this.arc(cx,cy,r,0,_2PI,false);
		this.cpath.push(this.PATH_CLOSE);
		this.addVectorElement(false,true);
		this.cpath = stack;
	},
	shapeCircle : function(cx,cy,r){
		var stack = this.cpath;
		this.cpath = [];
		this.arc(cx,cy,r,0,_2PI,false);
		this.cpath.push(this.PATH_CLOSE);
		this.addVectorElement(true,true);
		this.cpath = stack;
	},

	/* internal functions */
	addVectorElement : function(isfill,isstroke){}
});

// candle.svg.js

(function(){

/* ------------------- */
/*   SVG描画可能条件   */
/* ------------------- */
var SVGNS   = "http://www.w3.org/2000/svg",
	XLINKNS = "http://www.w3.org/1999/xlink";
if(!document.createElementNS || !document.createElementNS(SVGNS, 'svg').suspendRedraw){ return;}

/* ------------------------------------------- */
/*   VectorContext(SVG)クラス用const文字列集   */
/* ------------------------------------------- */
var S_PATH_MOVE   = ' M',
	S_PATH_LINE   = ' L',
	S_PATH_ARCTO  = ' A',
	S_PATH_CLOSE  = ' z',

	S_ATT_ID          = 'id',
	S_ATT_FILL        = 'fill',
	S_ATT_STROKE      = 'stroke',
	S_ATT_STROKEWIDTH = 'stroke-width',
	S_ATT_RENDERING   = 'shape-rendering',

	S_NONE = 'none',

	S_ANCHOR = { left:'start', center:'middle', right:'end'},
	S_HEIGHT = { top:-0.7, hanging:-0.6, middle:-0.25, alphabetic:0, bottom:0.1 };

/* ----------------- */
/*   SVG用ラッパー   */
/* ----------------- */
Candle.addTypes('svg');

Candle.addWrapper('svg:vector',{

	initialize : function(idname){
		Candle.wrapper.vector.prototype.initialize.call(this, idname);

		this.use = new Candle.TypeList('svg');

		// define const
		this.PATH_MOVE  = S_PATH_MOVE;
		this.PATH_LINE  = S_PATH_LINE;
		this.PATH_CLOSE = S_PATH_CLOSE;
		this.PATH_ARCTO = S_PATH_ARCTO;

		this.initElement();
	},

	/* additional functions (for initialize) */
	initElement : function(){
		var parent = this.canvas = _doc.getElementById(this.idname);

		var rect = Candle.getRectSize(this.canvas);
		var root = _doc.createElementNS(SVGNS,'svg');
		root.setAttribute('xmlns', SVGNS);
		root.setAttribute('xmlns:xlink', XLINKNS);
		root.setAttribute('id', this.canvasid);
		root.setAttribute('font-size', "10");
		root.setAttribute('font-family', "sans-serif");
		root.setAttribute('width', rect.width);
		root.setAttribute('height', rect.height);
		root.setAttribute('viewBox', [0,0,rect.width,rect.height].join(' '));
		parent.appendChild(root);

		this.child = root;
		this.afterInit();

		parent.toDataURL = function(type){
			return "data:image/svg+xml;base64," + window.btoa(root.outerHTML || new XMLSerializer().serializeToString(root));
		};
		parent.toBlob = function(){
			return new Blob([root.outerHTML || new XMLSerializer().serializeToString(root)], {type:'image/svg+xml'});
		};
	},

	initTarget : function(){
		this.target = _doc.getElementById(this.canvasid);
	},
	clear : function(){
		var root = this.canvas.firstChild, el = root.firstChild;
		while(!!el){ root.removeChild(el); el = root.firstChild;}

		this.resetElement();
	},

	/* layer functions */
	setLayerEdge : function(){ /* nop */ },
	createLayer : function(lid){
		var layer = _doc.createElementNS(SVGNS,'g');
		layer.setAttribute('id', lid);
		this.target.appendChild(layer);
		return layer;
	},
	getLayerById : function(id){
		return _doc.getElementById(id);
	},

	/* property functions */
	setRendering : function(render){
		this.target.setAttribute(S_ATT_RENDERING, render);
	},
	setUnselectable : function(unsel){
		unsel = ((unsel===(void 0)) ? true : !!unsel);
		this.canvas.style.MozUserSelect    = (unsel ? 'none' : 'text');
		this.canvas.style.WebkitUserSelect = (unsel ? 'none' : 'text');
		this.canvas.style.userSelect       = (unsel ? 'none' : 'text');
	},

	changeChildSize : function(child,width,height){
		child.setAttribute('width', width);
		child.setAttribute('height', height);
		var m = child.getAttribute('viewBox').split(/ /);
		child.setAttribute('viewBox', [m[0],m[1],width,height].join(' '));
	},

	/* Canvas API functions (for text) */
	fillText : function(text,x,y){
		var already = (!!this.vid && !!this.elements[this.vid]);
		var ME = Candle.ME;

		ME.style.font = this.font; ME.innerHTML = text;
		var top = y - (ME.offsetHeight * S_HEIGHT[this.textBaseline.toLowerCase()]);

		var el = (already ? this.elements[this.vid] : _doc.createElementNS(SVGNS,'text'));
		el.setAttribute('x', x);
		el.setAttribute('y', top);
		el.setAttribute(S_ATT_FILL, Candle.parse(this.fillStyle));
		el.setAttribute('text-anchor', S_ANCHOR[this.textAlign.toLowerCase()]);
		if(this.font.match(/(.+\s)?([0-9]+)px (.+)$/)){
			var style = RegExp.$1, size = RegExp.$2, family = RegExp.$3;
			el.setAttribute('font-size', size);
			
			if(!family.match(/^sans\-serif$/i)){ el.setAttribute('font-family', family);}
			else{ el.removeAttribute('font-family');}
			
			if(style.match(/(italic|oblique)/)){ el.setAttribute('font-style', RegExp.$1);}
			else{ el.removeAttribute('font-style');}
			
			if(style.match(/(bold|bolder|lighter|[1-9]00)/)){ el.setAttribute('font-weight', RegExp.$1);}
			else{ el.removeAttribute('font-weight');}
		}
		else{
			el.setAttribute('font', this.font);
		}
		if(!already){
			el.appendChild(_doc.createTextNode(text));
			this.target.appendChild(el);
			this.lastElement = el;
		}
		else{
			el.replaceChild(_doc.createTextNode(text), el.firstChild);
		}

		if(!already && !!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	},

	/* Canvas API functions (for image) */
	drawImage : function(image,sx,sy,sw,sh,dx,dy,dw,dh){
		if(sw===(void 0)){ sw=image.width; sh=image.height;}
		if(dx===(void 0)){ dx=sx; sx=0; dy=sy; sy=0; dw=sw; dh=sh;}
		var already = (!!this.vid && !!this.elements[this.vid]);

		var el = (already ? this.elements[this.vid] : _doc.createElementNS(SVGNS, "svg"));
		el.setAttribute("viewBox", [sx,sy,sw,sh].join(" "));
		el.setAttribute("x", dx);
		el.setAttribute("y", dy);
		el.setAttribute("width",  dw);
		el.setAttribute("height", dh);

		var img = (already ? el.firstChild : _doc.createElementNS(SVGNS, "image"));
		img.setAttributeNS(null, "width",  image.width);
		img.setAttributeNS(null, "height", image.height);
		img.setAttributeNS(XLINKNS, "xlink:href", image.src);
		if(!already){
			el.appendChild(img);
			this.target.appendChild(el);
			this.lastElement = el;
		}

		if(!already && !!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	},

	/* Canvas API functions (for transform) */
	translate : function(left,top){
		var child = this.canvas.firstChild;
		var m = child.getAttribute('viewBox').split(/ /);
		m[0]=-left, m[1]=-top;
		child.setAttribute('viewBox', m.join(' '));
	},

	/* extended functions */
	setDashSize : function(sizes){
		if(!this.lastElement){ return;}
		this.lastElement.setAttribute('stroke-dasharray', sizes.join(" "));
	},

	/* internal functions */
	addVectorElement : function(isfill,isstroke){
		var path = this.cpath.join(' ');

		var el = _doc.createElementNS(SVGNS,'path');
		el.setAttribute('d', path);
		el.setAttribute(S_ATT_FILL,   (isfill ? Candle.parse(this.fillStyle) : S_NONE));
		el.setAttribute(S_ATT_STROKE, (isstroke ? Candle.parse(this.strokeStyle) : S_NONE));
		if(isstroke) { el.setAttribute(S_ATT_STROKEWIDTH, this.lineWidth);}

		this.target.appendChild(el);
		this.lastElement = el;

		if(!!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	}
});

})();

// candle.canvas.js
 
(function(){

/* ---------------------- */
/*   canvas描画可能条件   */
/* ---------------------- */
if(!document.createElement('canvas').getContext){ return;}

/* -------------------------- */
/*   canvasブラウザ依存対策   */
/* -------------------------- */
if(window.CanvasRenderingContext2D){
	var p = CanvasRenderingContext2D.prototype;
	console.log(3);
	if(!p.setLineDash){
		if     ('mozDash' in p)       { p.setLineDash = function(sizes){ this.mozDash=sizes;};}
		else if('webkitLineDash' in p){ p.setLineDash = function(sizes){ this.webkitLineDash=sizes;};}
	}
}

/* -------------------- */
/*   Canvas用ラッパー   */
/* -------------------- */
Candle.addTypes('canvas');

Candle.addWrapper('canvas:wrapperbase',{

	initialize : function(idname){
		Candle.wrapper.wrapperbase.prototype.initialize.call(this, idname);

		// variables for internal
		this.context  = null;	// 本来のCanvasRenderingContext2Dオブジェクト

		this.use = new Candle.TypeList('canvas');

		this.x0 = 0;
		this.y0 = 0;

		this.initElement();
	},

	/* extend functions (initialize) */
	initElement : function(){
		var parent = this.canvas = _doc.getElementById(this.idname);

		var rect = Candle.getRectSize(parent);
		var root = _doc.createElement('canvas');
		root.id = this.canvasid;

		root.width  = rect.width;
		root.height = rect.height;
		root.style.width  = rect.width + 'px';
		root.style.height = rect.height + 'px';
		parent.appendChild(root);

		this.child = root;
		this.afterInit();
	},
	afterInit : function(){
		var parent = this.canvas;
		var child  = this.child;

		var self = this;
		parent.style.overflow = 'hidden';
		if(Candle.debugmode){
			parent.style.backgroundColor = "#efefef";
			parent.style.border = "solid 1px silver";
		}
		parent.getContext = function(type){ return self;};
		parent.toDataURL = function(type){
			return (!!type?child.toDataURL(type):child.toDataURL());
		};
		parent.toBlob = function(){
			try{ return child.toBlob();}catch(e){}
			/* Webkit, BlinkにtoBlobがない... */
			child.toDataURL().match(/data:(.*);base64,(.*)/);
			var bin = window.atob(RegExp.$2), len=bin.length;
			var buf = new Uint8Array(len);
			for(var i=0;i<len;i++){ buf[i]=bin.charCodeAt(i);}
			return new Blob([buf.buffer], {type:RegExp.$1});
		};
		child.toBlob = child.toBlob || child.msToBlob;

		this.setLayer();
		this.context = this.child.getContext('2d');

		Candle._initializing--;
		Candle.readyflag[this.idname] = true;
	},

	clear : function(){
		this.setProperties();
		this.context.setTransform(1,0,0,1,0,0); // 変形をリセット
		this.context.translate(this.x0, this.y0);
		var rect = Candle.getRectSize(this.canvas);
		this.context.clearRect(0,0,rect.width,rect.height);
	},

	/* layer functions */
	setLayer : function(layerid){
		this.currentLayerId = (!!layerid ? layerid : '_empty');
		if(this.isedgearray[this.currentLayerId] !== void 0)
			{ this.isedge = this.isedgearray[this.currentLayerId];}
		else
			{ this.isedge = this.isedgearray['_empty'];}
		this.setEdgeStyle(layerid);
	},
	setEdgeStyle : function(layerid){
		var s = this.canvas.style;
		if('imageRendering' in s){
			s.imageRendering = '';
			if(this.isedge){
				s.imageRendering = 'pixelated';
				if(!s.imageRendering){ s.imageRendering = '-webkit-optimize-contrast';}
				if(!s.imageRendering){ s.imageRendering = '-moz-crisp-edges';}
				if(!s.imageRendering){ s.imageRendering = '-o-crisp-edges';}
			}
		}
	},

	/* property functions */
	setRendering : function(render){
		this.isedgearray[this.currentLayerId] = (render==='crispEdges');
		this.isedge = this.isedgearray[this.currentLayerId];
		this.setEdgeStyle(this.currentLayerId);
	},
	setUnselectable : function(unsel){
		if(unsel===(void 0)){ unsel = true;}else{ unsel = !!unsel;}
		var s = this.canvas.style;
		s.MozUserSelect = s.WebkitUserSelect = s.userSelect = (unsel ? 'none' : 'text');
	},

	getContextElement : function(){ return this.child;},
	getLayerElement   : function(){ return this.child;},

	changeSize : function(width,height){
		var parent = this.canvas;
		parent.style.width  = width + 'px';
		parent.style.height = height + 'px';

		var child = this.child;
		var left = parseInt(child.style.left), top = parseInt(child.style.top);
		width += (left<0?-left:0);
		height += (top<0?-top:0);
		child.style.width  = width + 'px';
		child.style.height = height + 'px';
		child.width  = width;
		child.height = height;
	},

	/* 内部用関数 */
	setProperties : function(){
		var c = this.context;
		c.fillStyle    = this.fillStyle;
		c.strokeStyle  = this.strokeStyle;
		c.lineWidth    = this.lineWidth;
		c.font         = this.font;
		c.textAlign    = this.textAlign;
		c.textBaseline = this.textBaseline;
	},

	/* Canvas API functions (for path) */
	beginPath : function(){ this.context.beginPath();},
	closePath : function(){ this.context.closePath();},

	moveTo : function(x,y){
		this.context.moveTo(x,y);
	},
	lineTo : function(x,y){
		this.context.lineTo(x,y);
	},
	rect : function(x,y,w,h){
		this.context.rect(x,y,w,h);
	},
	arc  : function(cx,cy,r,startRad,endRad,antiClockWise){
		this.context.arc(cx,cy,r,startRad,endRad,antiClockWise);
	},

	/* Canvas API functions (for drawing) */
	fill       : function(){ this.setProperties(); this.context.fill();},
	stroke     : function(){ this.setProperties(); this.context.stroke();},
	shape : function(){
		this.setProperties();
		this.context.fill();
		this.context.stroke();
	},

	/* Canvas API functions (rect) */
	fillRect   : function(x,y,w,h){
		this.setProperties();
		this.context.fillRect(x,y,w,h);
	},
	strokeRect : function(x,y,w,h){
		this.setProperties();
		this.context.strokeRect(x,y,w,h);
	},
	shapeRect : function(x,y,w,h){
		this.setProperties();
		this.context.fillRect(x,y,w,h);
		this.context.strokeRect(x,y,w,h);
	},

	/* Canvas API functions (for text) */
	fillText : function(text,x,y){
		this.setProperties();
		this.context.fillText(text,x,y);
	},

	/* Canvas API functions (for image) */
	drawImage : function(){
		this.context.drawImage.apply(this.context, arguments);
	},

	/* Canvas API functions (for transform) */
	translate : function(left,top){
		this.x0 = left;
		this.y0 = top;
		this.context.translate(left, top);
	},

	/* extended functions */
	setLinePath : function(){
		var _args=arguments, _len=_args.length, len=_len-((_len|1)?1:2), a=[];
		for(var i=0;i<len;i+=2){ a[i>>1] = [_args[i],_args[i+1]];}
		this.setLinePath_com.call(this,a);
		if(_args[_len-1]){ this.context.closePath();}
	},
	setOffsetLinePath : function(){
		var _args=arguments, _len=_args.length, len=_len-((_len|1)?1:2)-2, a=[];
		for(var i=0;i<len;i+=2){ a[i>>1] = [_args[i+2]+_args[0], _args[i+3]+_args[1]];}
		this.context.beginPath();
		this.setLinePath_com.call(this,a);
		if(_args[_len-1]){ this.context.closePath();}
	},
	setLinePath_com : function(array){
		for(var i=0,len=array.length;i<len;i++){
			var ar=array[i];
			if(i===0){ this.context.moveTo(ar[0],ar[1]);}
			else     { this.context.lineTo(ar[0],ar[1]);}
		}
	},
	setDashSize : function(size){ },

	strokeLine : function(x1,y1,x2,y2){
		var c = this.context;
		this.setProperties();
		c.beginPath();
		c.moveTo(x1,y1);
		c.lineTo(x2,y2);
		c.stroke();
	},
	strokeDashedLine : function(x1,y1,x2,y2,sizes){
		var self = this, c = this.context;
		this.strokeDashedLine = ((!!this.context.setLineDash) ?
			function(x1,y1,x2,y2,sizes){
				var c = self.context;
				self.setProperties();
				c.beginPath();
				c.moveTo(x1,y1);
				c.lineTo(x2,y2);
				c.setLineDash(sizes);
				c.stroke();
				c.setLineDash([]);
			}
		:
			function(x1,y1,x2,y2,sizes){
				if((sizes.length%2)===1){ sizes = sizes.concat(sizes);}
				var length = Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
				var distance=0, phase=0, tilt=(y2-y1)/(x2-x1), tilts=tilt*tilt+1;
				var c = self.context;
				self.setProperties();
				c.beginPath();
				c.moveTo(x1, y1);
				while(distance<length){
					var a = Math.sqrt(distance*distance/tilts);
					var px = x1+a, py = y1+tilt*a;
					if((phase&1)===0){ c.moveTo(px, py);}
					else             { c.lineTo(px, py);}
					distance += sizes[phase];
					phase++;
					if(phase>=sizes.length){ phase=0;}
				}
				c.stroke();
			}
		);
		this.strokeDashedLine(x1,y1,x2,y2,sizes);
	},
	strokeCross : function(cx,cy,l){
		var c = this.context;
		this.setProperties();
		c.beginPath();
		c.moveTo(cx-l,cy-l);
		c.lineTo(cx+l,cy+l);
		c.moveTo(cx-l,cy+l);
		c.lineTo(cx+l,cy-l);
		c.stroke();
	},

	/* extended functions (circle) */
	fillCircle : function(cx,cy,r){
		var c = this.context;
		this.setProperties();
		c.beginPath();
		c.arc(cx,cy,r,0,_2PI,false);
		c.fill();
	},
	strokeCircle : function(cx,cy,r){
		var c = this.context;
		this.setProperties();
		c.beginPath();
		c.arc(cx,cy,r,0,_2PI,false);
		c.stroke();
	},
	shapeCircle : function(cx,cy,r){
		var c = this.context;
		this.setProperties();
		c.beginPath();
		c.arc(cx,cy,r,0,_2PI,false);
		c.fill();
		c.stroke();
	}
});

})();

// candle.vml.js
 
(function(){

/* ------------------- */
/*   VML描画可能条件   */
/* ------------------- */
try{ document.namespaces.add("v", "urn:schemas-microsoft-com:vml");}catch(e){ return;}

/* ------------------------------------------- */
/*   VectorContext(VML)クラス用const文字列集   */
/* ------------------------------------------- */
var V_TAG_SHAPE    = '<v:shape',
	V_TAG_GROUP    = '<v:group',
	V_TAG_IMAGE    = '<v:image',
	V_TAG_TEXTPATH = '<v:textpath',
	V_TAG_POLYLINE = '<v:polyline',
	V_TAG_PATH_FOR_TEXTPATH = '<v:path textpathok="t" />',
	V_EL_UNSELECTABLE = '', // デフォルトはunselectableでない
//	V_EL_UNSELECTABLE = ' unselectable="on"',
	V_TAGEND      = '>',
	V_TAGEND_NULL = ' />',
	V_CLOSETAG_SHAPE    = '</v:shape>',
	V_CLOSETAG_GROUP    = '</v:group>',
	V_CLOSETAG_IMAGE    = '</v:image>',
	V_CLOSETAG_TEXTPATH = '</v:textpath>',
	V_CLOSETAG_POLYLINE = '</v:polyline>',

	V_ATT_ID     = ' id="',
	V_ATT_PATH   = ' path="',
	V_ATT_POINTS = ' points="',
	V_ATT_STYLE  = ' style="',
	V_ATT_STRING = ' string="',
	V_ATT_COORDSIZE    = ' coordsize="100,100"',
	V_ATT_FILLCOLOR    = ' fillcolor="',
	V_ATT_STROKECOLOR  = ' strokecolor="',
	V_ATT_STROKEWEIGHT = ' strokeweight="',
	V_ATT_END = '"',
	V_ATT_STYLE_TEXTBOX = ' style="white-space:nowrap;cursor:default;font:10px sans-serif;"',
	V_DEF_ATT_POLYLINE  = ' stroked="f" filled="t"',
	V_DEF_ATT_TEXTPATH  = ' on="t" xscale="t"',

	V_STYLE_LEFT  = 'left:',
	V_STYLE_TOP   = 'top:',
	V_STYLE_FONT  = 'font:',
	V_STYLE_ALIGN = 'v-text-align:',
	V_STYLE_END   = ';',

	V_PATH_MOVE   = ' m',
	V_PATH_LINE   = ' l',
	V_PATH_CLOSE  = ' x',
	V_PATH_ARCTO  = ' ?', /* 'at'か'wa'どちらか */
	V_PATH_NOSTROKE = ' ns',
	V_PATH_NOFILL   = ' nf',

	V_WIDTH = { left:0, center:0.5, right:1 },
	V_HEIGHT = { top:-0.7, hanging:-0.66, middle:-0.3, alphabetic:0, bottom:0.1 },

	Z  = 10,
	Z2 = Z/2;

/* ----------------------- */
/*   CSS, NameSpace設定    */
/* ----------------------- */
/* addNameSpace for VML */
/* ネームスペースの追加はVML描画可能条件に移動しました */

/* addStyleSheet for VML */
var V_BH  = "behavior: url(#default#VML);";
var V_BH2 = V_BH + " position:absolute; width:10px; height:10px;";
Candle.addCSS("v\\:shape",    V_BH2);
Candle.addCSS("v\\:group",    V_BH2);
Candle.addCSS("v\\:polyline", V_BH2);
Candle.addCSS("v\\:image",    V_BH2);
Candle.addCSS("v\\:path",     V_BH);
Candle.addCSS("v\\:textpath", V_BH);
Candle.addCSS("v\\:stroke",   V_BH);

/* ----------------------- */
/*   VectorContextクラス   */
/* ----------------------- */
Candle.addTypes('vml');

Candle.addWrapper('vml:vector',{

	initialize : function(idname){
		Candle.wrapper.vector.prototype.initialize.call(this, idname);

		this.use = new Candle.TypeList('vml');

		// define const
		this.PATH_MOVE  = V_PATH_MOVE;
		this.PATH_LINE  = V_PATH_LINE;
		this.PATH_CLOSE = V_PATH_CLOSE;
		this.PATH_ARCTO = V_PATH_ARCTO;

		this.initElement();
	},

	/* additional functions (for initialize) */
	initElement : function(){
		var parent = this.canvas = _doc.getElementById(this.idname);
		parent.style.display  = 'block';
		parent.style.position = 'relative';

		var rect = Candle.getRectSize(this.canvas);
		var root = _doc.createElement('div');
		root.id = this.canvasid;
		root.style.position = 'absolute';
		root.style.left   = '-2px';
		root.style.top    = '-2px';
		root.style.width  = rect.width + 'px';
		root.style.height = rect.height + 'px';
		this.canvas.appendChild(root);

		this.child = root;
		this.afterInit();
	},

	initTarget : function(){
		this.target = _doc.getElementById(this.canvasid);
	},
	clear : function(){
		var root = this.canvas.firstChild, el = root.firstChild;
		while(!!el){ root.removeChild(el); el=root.firstChild;}

		this.resetElement();
	},

	/* layer functions */
	createLayer : function(lid){
		var layer = _doc.createElement('div');
		layer.id = lid;
		layer.unselectable = (!!V_EL_UNSELECTABLE ? 'on' : '');
		layer.style.position = 'absolute';
		layer.style.left   = '0px';
		layer.style.top    = '0px';
		this.target.appendChild(layer);
		return layer;
	},
	getLayerById : function(id){
		return _doc.getElementById(id);
	},

	/* property functions */
	setRendering : function(render){
		this.isedgearray[this.currentLayerId] = (render==='crispEdges');
		this.isedge = this.isedgearray[this.currentLayerId];
	},
	setUnselectable : function(unsel){
		if(unsel===(void 0)){ unsel = true;}else{ unsel = !!unsel;}
		V_EL_UNSELECTABLE = (unsel ? ' unselectable="on"' : '');
		this.canvas.unselectable = (unsel ? 'on' : '');
		this.child.unselectable  = (unsel ? 'on' : '');
	},

	changeChildSize : function(child,width,height){
		child.style.width  = width + 'px';
		child.style.height = height + 'px';
	},

	/* Canvas API functions (for path) */
	moveTo : function(x,y){
		this.cpath.push(this.PATH_MOVE,this.ePos(x,true),this.ePos(y,true));
		this.lastpath = this.PATH_MOVE;
	},
	lineTo : function(x,y){
		if(this.lastpath!==this.PATH_LINE){ this.cpath.push(this.PATH_LINE);}
		this.cpath.push(this.ePos(x,true),this.ePos(y,true));
		this.lastpath = this.PATH_LINE;
	},
	rect : function(x,y,w,h){
		x=this.ePos(x,true); y=this.ePos(y,true); w=this.eLen(w); h=this.eLen(h);
		this.cpath.push(this.PATH_MOVE,x,y,this.PATH_LINE,(x+w),y,(x+w),(y+h),x,(y+h),this.PATH_CLOSE);
		this.lastpath = this.PATH_CLOSE;
	},
	arc : function(cx,cy,r,startRad,endRad,antiClockWise){
		var sx,sy,ex,ey;
		if(endRad-startRad>=_2PI){ sx=cx+r, sy=cy, ex=cx+r, ey=cy;}
		else{
			sx = cx + r*Math.cos(startRad), sy = cy + r*Math.sin(startRad),
			ex = cx + r*Math.cos(endRad),   ey = cy + r*Math.sin(endRad);
		}
		cx=(cx*Z-Z2)|0, cy=(cy*Z-Z2)|0, r=(r*Z)|0;
		sx=(sx*Z-Z2)|0, sy=(sy*Z-Z2)|0, ex=(ex*Z-Z2)|0, ey=(ey*Z-Z2)|0;
		var com = (antiClockWise ? 'at' : 'wa');
		if(endRad-startRad>=_2PI){ sx+=1;}
		this.cpath.push(com,(cx-r),(cy-r),(cx+r),(cy+r),sx,sy,ex,ey);
		this.lastpath = com;
	},

	/* Canvas API functions (for text) */
	fillText : function(text,x,y){
		var already = (!!this.vid && !!this.elements[this.vid]);
		var ME = Candle.ME;

		x=(x*Z-Z2)|0, y=(y*Z-Z2)|0;
		ME.style.font = this.font; ME.innerHTML = text;
		var top  = y - ((ME.offsetHeight * V_HEIGHT[this.textBaseline.toLowerCase()])*Z-Z2)|0;

		var wid = (ME.offsetWidth*Z-Z2)|0;
		var left = x - (wid * V_WIDTH[this.textAlign.toLowerCase()])|0;

		if(!already){
			var ar = [
				V_TAG_GROUP, V_ATT_COORDSIZE, V_TAGEND,
					V_TAG_POLYLINE, V_ATT_POINTS, [left,top,left+wid,top].join(','), V_ATT_END,
					V_DEF_ATT_POLYLINE, V_ATT_FILLCOLOR, Candle.parse(this.fillStyle), V_ATT_END, V_TAGEND,
						V_TAG_PATH_FOR_TEXTPATH,
						
						V_TAG_TEXTPATH, V_DEF_ATT_TEXTPATH, V_ATT_STRING, text, V_ATT_END,
						V_ATT_STYLE, V_STYLE_FONT, this.font, V_STYLE_END,
						V_STYLE_ALIGN, this.textAlign, V_STYLE_END, V_ATT_END, V_TAGEND_NULL,
					V_CLOSETAG_POLYLINE,
				V_CLOSETAG_GROUP
			];

			this.target.insertAdjacentHTML('BeforeEnd', ar.join(''));
			this.lastElement = this.target.lastChild.lastChild;
		}
		else{
			var el = this.elements[this.vid];
//			el.points = [left,top,left+wid,top].join(',');
			el.fillcolor = Candle.parse(this.fillStyle);
			el.lastChild.style.font = this.font;
			el.lastChild.string = text;
		}

		if(!already && !!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	},

	/* Canvas API functions (for image) */
	drawImage : function(image,sx,sy,sw,sh,dx,dy,dw,dh){
		if(sw===(void 0)){ sw=image.width; sh=image.height;}
		if(dx===(void 0)){ dx=sx; sx=0; dy=sy; sy=0; dw=sw; dh=sh;}
		var already = (!!this.vid && !!this.elements[this.vid]);

		var el;
		if(!already){
			var ar = [V_TAG_IMAGE, ' src="', image.src, V_ATT_END, V_ATT_COORDSIZE, V_TAGEND_NULL];
			this.target.insertAdjacentHTML('BeforeEnd', ar.join(''));
			this.lastElement = this.target.lastChild;
			el = this.lastElement;
		}
		else{
			el = this.elements[this.vid];
			el.src = image.src;
		}
		el.style.left = dx;
		el.style.top  = dy;
		el.style.width  = dw;
		el.style.height = dh;
		el.cropleft = sx/image.width;
		el.croptop  = sy/image.height;
		el.cropright  = (1-(sx+sw)/image.width);
		el.cropbottom = (1-(sy+sh)/image.height);

		if(!already && !!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	},

	/* Canvas API functions (for transform) */
	translate : function(left,top){
		var child = this.canvas.firstChild;
		child.style.position = 'absolute';
		child.style.left = left+'px';
		child.style.top  = top +'px';
	},

	/* extended functions */
	setLinePath_com : function(array){
		this.cpath = [];
		for(var i=0,len=array.length;i<len;i++){
			this.cpath.push(i===0 ? this.PATH_MOVE : this.PATH_LINE);
			this.cpath.push(this.ePos(array[i][0],true),this.ePos(array[i][1],true));
		}
	},
	setDashSize : function(sizes){
		if(!this.lastElement){ return;}
		var el = _doc.createElement('v:stroke');
		if     (sizes[0]<=2){ el.dashstyle = 'ShortDash';}
		else if(sizes[0]<=5){ el.dashstyle = 'Dash';}
		else                { el.dashstyle = 'LongDash';}
		this.lastElement.appendChild(el);
	},

	strokeLine : function(x1,y1,x2,y2){
		x1=this.ePos(x1,true); y1=this.ePos(y1,true); x2=this.ePos(x2,true); y2=this.ePos(y2,true);
		Candle.wrapper.vector.prototype.strokeLine.call(this,x1,y1,x2,y2);
	},
	strokeDashedLine : function(x1,y1,x2,y2,sizes){
		x1=this.ePos(x1,true); y1=this.ePos(y1,true); x2=this.ePos(x2,true); y2=this.ePos(y2,true);
		Candle.wrapper.vector.prototype.strokeDashedLine.call(this,x1,y1,x2,y2,sizes);
	},
	strokeCross : function(cx,cy,l){
		cx=this.ePos(cx,true); cy=this.ePos(cy,true); l=this.eLen(l);
		Candle.wrapper.vector.prototype.strokeCross.call(this,cx,cy,l);
	},

	/* internal functions */
	ePos : function(num,stroke){
		if(!stroke){ num = (num+(num>0?0.5:-0.5))|0;}
		else       { num = ((num+(num>0?0.5:-0.5) - (this.lineWidth%2===1?0.5:0))|0);}
		return (num*Z-Z2)|0;
	},
	eLen : function(num){
		return (num*Z)|0;
	},
	addVectorElement : function(isfill,isstroke){
		var path = this.cpath.join(' ');

		path = [path, (!isfill ? V_PATH_NOFILL : ''), (!isstroke ? V_PATH_NOSTROKE : '')].join('');
		var ar = [V_TAG_SHAPE, V_EL_UNSELECTABLE, V_ATT_COORDSIZE, V_ATT_PATH, path, V_ATT_END];
		if(isfill)  { ar.push(V_ATT_FILLCOLOR, Candle.parse(this.fillStyle), V_ATT_END);}
		if(isstroke){ ar.push(V_ATT_STROKECOLOR, Candle.parse(this.strokeStyle), V_ATT_END, V_ATT_STROKEWEIGHT, this.lineWidth, 'px', V_ATT_END);}
		ar.push(V_TAGEND_NULL);

		this.target.insertAdjacentHTML('BeforeEnd', ar.join(''));
		this.lastElement = this.target.lastChild;

		if(!!this.vid){ this.elements[this.vid] = this.lastElement; this.vid='';}
	}
});

})();

// outro.js

})();
